@@ -36,8 +36,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 /**
@@ -70,8 +70,6 @@
  * gst-launch-1.0 jackaudiosrc connect=0 ! jackaudiosink connect=0
  * ]| Get audio input into gstreamer from jack.
  * </refsect2>
- *
- * Last reviewed on 2008-07-22 (0.10.4)
  */
 
 #ifdef HAVE_CONFIG_H
@@ -82,6 +80,8 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include <gst/audio/audio.h>
+
 #include "gstjackaudiosrc.h"
 #include "gstjackringbuffer.h"
 #include "gstjackutil.h"
@@ -469,9 +469,15 @@ gst_jack_ring_buffer_acquire (GstAudioRingBuffer * buf,
     /* find all the physical output ports. A physical output port is a port
      * associated with a hardware device. Someone needs connect to a physical
      * port in order to capture something. */
-    ports =
-        jack_get_ports (client, NULL, NULL,
-        JackPortIsPhysical | JackPortIsOutput);
+
+    if (src->port_pattern == NULL) {
+      ports = jack_get_ports (client, NULL, NULL,
+          JackPortIsPhysical | JackPortIsOutput);
+    } else {
+      ports = jack_get_ports (client, src->port_pattern, NULL,
+          JackPortIsOutput);
+    }
+
     if (ports == NULL) {
       /* no ports? fine then we don't do anything except for posting a warning
        * message. */
@@ -680,7 +686,7 @@ enum
 #define DEFAULT_PROP_SERVER 		NULL
 #define DEFAULT_PROP_CLIENT_NAME	NULL
 #define DEFAULT_PROP_TRANSPORT	GST_JACK_TRANSPORT_AUTONOMOUS
-
+#define DEFAULT_PROP_PORT_PATTERN     	NULL
 enum
 {
   PROP_0,
@@ -688,11 +694,11 @@ enum
   PROP_SERVER,
   PROP_CLIENT,
   PROP_CLIENT_NAME,
+  PROP_PORT_PATTERN,
   PROP_TRANSPORT,
   PROP_LAST
 };
 
-
 /* the capabilities of the inputs and outputs.
  *
  * describe the real formats here.
@@ -756,11 +762,9 @@ gst_jack_audio_src_class_init (GstJackAudioSrcClass * klass)
           DEFAULT_PROP_SERVER, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   /**
-   * GstJackAudioSrc:client-name
+   * GstJackAudioSrc:client-name:
    *
    * The client name to use.
-   *
-   * Since: 0.10.31
    */
   g_object_class_install_property (gobject_class, PROP_CLIENT_NAME,
       g_param_spec_string ("client-name", "Client name",
@@ -773,13 +777,23 @@ gst_jack_audio_src_class_init (GstJackAudioSrcClass * klass)
           GST_TYPE_JACK_CLIENT,
           GST_PARAM_MUTABLE_READY | G_PARAM_READWRITE |
           G_PARAM_STATIC_STRINGS));
+   /**
+    * GstJackAudioSrc:port-pattern
+    *
+    * autoconnect to ports matching pattern, when NULL connect to physical ports
+    *
+    * Since: 1.6
+    */
+  g_object_class_install_property (gobject_class, PROP_PORT_PATTERN,
+      g_param_spec_string ("port-pattern", "port pattern",
+          "A pattern to select which ports to connect to (NULL = first physical ports)",
+          DEFAULT_PROP_PORT_PATTERN,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   /**
-   * GstJackAudioSink:transport
+   * GstJackAudioSink:transport:
    *
    * The jack transport behaviour for the client.
-   *
-   * Since: 0.10.31
    */
   g_object_class_install_property (gobject_class, PROP_TRANSPORT,
       g_param_spec_flags ("transport", "Transport mode",
@@ -832,6 +846,11 @@ gst_jack_audio_src_dispose (GObject * object)
     src->client_name = NULL;
   }
 
+  if (src->port_pattern != NULL) {
+    g_free (src->port_pattern);
+    src->port_pattern = NULL;
+  }
+
   G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
@@ -846,6 +865,10 @@ gst_jack_audio_src_set_property (GObject * object, guint prop_id,
       g_free (src->client_name);
       src->client_name = g_value_dup_string (value);
       break;
+    case PROP_PORT_PATTERN:
+      g_free (src->port_pattern);
+      src->port_pattern = g_value_dup_string (value);
+      break;
     case PROP_CONNECT:
       src->connect = g_value_get_enum (value);
       break;
@@ -878,6 +901,9 @@ gst_jack_audio_src_get_property (GObject * object, guint prop_id,
     case PROP_CLIENT_NAME:
       g_value_set_string (value, src->client_name);
       break;
+    case PROP_PORT_PATTERN:
+      g_value_set_string (value, src->port_pattern);
+      break;
     case PROP_CONNECT:
       g_value_set_enum (value, src->connect);
       break;
